'EVE2 driver for Newhaven LCD display modules by Raymond Allen  
'MIT license

OBJ
  SPI:  "SPI_Asm_RJA2"  '"SPI_Spin"'   'You can either use Assembly or Spin (Spin is slower, but saves a cog)

VAR
  long  pin_sck, pin_miso, pin_mosi, pin_csn 'SPI interface pins
  long  pin_intn, pin_pdn 'EVE interrupt and power down pins
  long  CommandPointer  'Points to current position in command buffer.  This gets added to RAM_CMD
  long  DisplayListPointer 'Pointer to current position in display list (for next direct write to display list)

CON     InitBrightness=128/2  'Initial display brightness 128=max, 0=off

CON 'Settings for Newhaven 7" WVGA display module with eve2
        DispWidth = 800
        DispHeight = 480
        DispHCycle =  928
        DispHOffset = 88
        DispHSync0 = 0
        DispHSync1 = 48
        DispVCycle = 525
        DispVOffset = 32
        DispVSync0 = 0
        DispVSync1 = 3
        DispPCLK = 2
        DispSwizzle = 0
        DispPCLKPol = 1
        DispCSpread = 0
        DispDither = 1

PUB Start(sck,miso,mosi,csn,intn,pdn)|i,j 'initialize SPI interface and activate EVE given all pins
    'Assing pins to those given
     pin_sck:=sck
     pin_miso:=miso
     pin_mosi:=mosi
     pin_csn:=csn
     pin_intn:=intn
     pin_pdn:=pdn
      
    'We will cycle the power down pin to force reset of EVE2
    high(pin_csn)
    low(pin_pdn)
    waitcnt(cnt+clkfreq/10)
    high(pin_pdn)
    waitcnt(cnt+clkfreq/10)

    '' Initialize SPI Engine with Clock Delay of 15 and Clock State of 0 
    SPI.start(10,0)             

    'Send Active command
    low(pin_csn)
    SPI.SHIFTOUT(pin_mosi, pin_sck, SPI#MSBFIRST , 24, 0)   'send address + dummy byte
    high(pin_csn)
    low(pin_csn)
    SPI.SHIFTOUT(pin_mosi, pin_sck, SPI#MSBFIRST , 24, 0)   'send address + dummy byte
    high(pin_csn)
    
    'Wait until ID reads properly
    repeat          
      i:=$302000  'ID_Reg
      j:=ReadReg8(i)
      waitcnt(cnt+clkfreq/10)
    until j==$7C

    'Set display vaules
    wr16(REG_HCYCLE, DispHCycle)
    wr16(REG_HOFFSET, DispHOffset)
    wr16(REG_HSYNC0, DispHSync0)
    wr16(REG_HSYNC1, DispHSync1)
    wr16(REG_VCYCLE, DispVCycle)
    wr16(REG_VOFFSET, DispVOffset)
    wr16(REG_VSYNC0, DispVSync0)
    wr16(REG_VSYNC1, DispVSync1)
    wr8(REG_SWIZZLE, DispSwizzle)
    wr8(REG_PCLK_POL, DispPCLKPol)
    wr8(REG_CSPREAD, DispCSpread)
    wr16(REG_HSIZE, DispWidth)
    wr16(REG_VSIZE, DispHeight)

    'Adjust brightness
    wr8(REG_PWM_DUTY, InitBrightness)  '128=max, 0=off

    'Enable display bit
    WR8(REG_GPIO_DIR,RD8(REG_GPIO_DIR)|$80)
    WR8(REG_GPIO,RD8(REG_GPIO)|$80)

    'Set pixel clock
    WR8(REG_PCLK,DispPCLK)

    'Initialize co-processor commmand pointer
    CommandPointer:=0  

PUB SetBrightness(Brightness) 'set LCD backlight brightness:  128=max, 0=off
  wr8(REG_PWM_DUTY, Brightness)  '128=max, 0=off

PUB StartDisplayList 'reset display list pointer to beginning of display list
  DisplayListPointer:=RAM_DL

PUB ClearScreen(R,G,B)  'Clear the screen to specified color   
  WrDl32(Clear_Color_RGB(r,g,b)) 'set color
  WrDl32(CLEAR(1, 1, 1)) 'clear everything

  
PUB EndDisplayList  'send completed display list to screen
  WrDl32(DISPLAY)  'mark end of display list
  wr8(REG_DLSWAP,%10) 'update after current frame

PUB DrawCharacter(x, y, font, char) 'Draw character char using font #font to x,y location (x,y limited to 0..511)
  WrDl32(VERTEX2II(x, y, font, char))

PUB BeginDrawingBitmaps  'Start list of bitmaps to draw
  WrDl32(BeginBitmaps)  'start drawing bitmaps
  
PUB EndBegin  'mark end of "Begin" section
  WrDl32(End)  'end drawing bitmaps
  
PUB SetColor(r, g, b) 'Set color for subsequent operations to RGB(r,g,b)
  WrDl32(COLOR_RGB(r, g, b))

PUB SetPointSize(sz) 'set point size to 20 pixels in radius (units of 1/16 pixel, 20*16=320)         
  WrDl32(POINT_SIZE(sz<<4))

PUB TranslateX(x)   'shift subsequent points by x/16 pixels in x direction
  WrDl32(Vertex_Translate_X(x<<4))

PUB TranslateY(y)   'shift subsequent points by y/16 pixels in x direction
  WrDl32(Vertex_Translate_Y(y<<4))

PUB BeginDrawingPoints  'Start list of points to draw
  WrDl32(VertexFormat(4)) 'Select vertex format 4 (1/16 pixel precision, range -1024 to 1023) 
  WrDl32(BEGINPOINTS)

PUB DrawPoint(x,y)  'draw a point at x,y in units of 1/16 pixel
  WrDl32(VERTEX2F(x<<4, y<<4))  
    
PUB StartCmdDisplayList 'Start a new display list using the Co-Processor
  CMD(CMD_DLSTART)

PUB CmdClearColor(r,g,b) 'set clear screen color to RGB(r,g,b)
  CMD(CLEAR_COLOR_RGB(r, g, b))

PUB CmdClearScreen 'clear screen to screen clear color
  CMD(Clear(1,1,1))

PUB CmdButton(x,y,w,h,font,options,pString) 'draw button at x,y with dimensions w,h using font# font, options and pointer to string
  button(x,y,w,h,font,options,pString)     

PUB CmdClock(x,y,r,options,h,m,s,ms) 'draw a clock at x,y with radius of r and options at time set to h hours, m minutes, s seconds, ms milliseconds
  Clock(x,y,r,options,h,m,s,ms)

PUB EndCmdDisplayList 'End and show display list using the Co-Processor
  CMD(DISPLAY)
  CMD(CMD_DLSWAP)

PUB CmdGuage(x,y,r,options,major,minor,val,range)  'draw an analog guage at x,y with radius r, # of major and minor ticks, value and range
  Gauge(x,y,r,options,major,minor,val,range)      
    
PUB CmdRomFont(font,romslot)'Assign font# font to slot# romslot (needed only to use the biggest fonts, >#31) 
  ROMFONT(font,romslot)

PUB CmdText(x,y,font,options,pString) 'Draw text at x,y using font# font, options and provided string pointer   
  Text(x,y,font,options,pString)
  
CON 'EVE Registers and addresses
        REG_HCYCLE =    $30202C
        REG_HOFFSET =   $302030
        REG_HSYNC0 =    $302038
        REG_HSYNC1 =    $30203C
        REG_VCYCLE =    $302040
        REG_VOFFSET =   $302044
        REG_VSYNC0 =    $30204C
        REG_VSYNC1 =    $302050
        REG_SWIZZLE =   $302064
        REG_PCLK_POL =  $30206C
        REG_CSPREAD=    $302068
        REG_HSIZE =     $302034
        REG_VSIZE =     $302048
        REG_GPIO_DIR =  $302090
        REG_GPIO =      $302094
        REG_PCLK =      $302070
        REG_DLSWAP =    $302054
        RAM_DL =        $300000
        RAM_CMD =       $308000
        REG_PWM_DUTY =  $3020D4
        REG_CMD_WRITE = $3020FC
        
CON  'Co-processor commands
        CMD_DLSTART =  $ffffff00
        CMD_DLSWAP =   $ffffff01
        CMD_BUTTON =   $ffffff0d
        CMD_CLOCK =    $ffffff14
        CMD_GAUGE =    $ffffff13
        CMD_ROMFONT =  $ffffff3f
        CMD_TEXT =     $ffffff0c

PUB CMD(command)
   wr32(RAM_CMD+CommandPointer,command)
   commandPointer+=4
   CommandPointer&=$FFF  '4k circular buffer
   wr16(REG_CMD_WRITE,CommandPointer)
   
PUB Button(x,y,w,h,font,options,pString)|i,j
  CMD(CMD_BUTTON)
  CMD(y<<16+x)
  CMD(h<<16+w)
  CMD(options<<16+font)
  j:=(strsize(pString)+4)/4 
  repeat i from 1 to j
    CMD(byte[pString+3]<<24+byte[pString+2]<<16+byte[pString+1]<<8+byte[pString])
    pString+=4

PUB Text(x,y,font,options,pString)|i,j
  CMD(CMD_TEXT)
  CMD(y<<16+x)
  CMD(options<<16+font)
  j:=(strsize(pString)+4)/4 
  repeat i from 1 to j
    CMD(byte[pString+3]<<24+byte[pString+2]<<16+byte[pString+1]<<8+byte[pString])
    pString+=4

PUB Clock(x,y,r,options,h,m,s,ms)
  CMD(CMD_CLOCK)
  CMD(y<<16+x)
  CMD(options<<16+r)
  CMD(m<<16+h)
  CMD(ms<<16+s)

PUB Gauge(x,y,r,options,major,minor,val,range)
  CMD(CMD_GAUGE)
  CMD(y<<16+x)
  CMD(options<<16+r)
  CMD(minor<<16+major)
  CMD(range<<16+val)   

PUB ROMFONT(font,romslot)
  CMD(CMD_ROMFONT)
  CMD(font)
  CMD(romslot)

PUB Clear_Color_RGB(r,b,g)
  return $02<<24+r<<16+b<<8+g
PUB Color_RGB(r,b,g)
  return $04<<24+r<<16+b<<8+g  
PUB Clear(c,s,t) 'c=1 to clear color buffer, s=1 to clear stencil buffer, t=1 to clear tag buffer
  return $26<<24+(c&1)<<2+(s&1)<<1+(t&1)
PUB Display
  return 0
PUB Vertex2II(x,y,handle,cell)
  return $2<<30+x<<21+y<<12+handle<<7+cell
PUB Vertex2F(x,y)
  return $1<<30+x<<15+y  
PUB Vertex_Translate_X(dx)
  return ($2B<<24+dx)
PUB Vertex_Translate_Y(dy)
  return ($2C<<24+dy)    
PUB BeginBitmaps
  return $1F<<24+1
PUB End
  return $21<<24
PUB Point_Size(size)
  return $0D<<24+size
PUB BeginPoints
  return $1F<<24+2
PUB VertexFormat(frac)
  return $27<<24+frac  

PRI WrDl32(data) 'Write 32 bits to display list
  WriteReg32(DisplayListPointer,data)
  DisplayListPointer+=4


PRI WR16(address,data)
  WriteReg16(address,data)
PRI WR8(address,data)
  WriteReg8(address,data)
PRI WR32(address,data)
  WriteReg32(address,data)
PRI RD8(address)
  return ReadReg8(address)  

PRI WriteReg8(Address,data)  '8 bits
  address|=$800000
  low(pin_csn)
  SPI.SHIFTOUT(pin_mosi, pin_sck, SPI#MSBFIRST , 24, address)   'send address
  SPI.SHIFTOUT(pin_mosi, pin_sck, SPI#MSBFIRST , 8, data)   'send data
  high(pin_csn)

PRI WriteReg16(Address,data) '16 bits
  address|=$800000
  low(pin_csn)
  SPI.SHIFTOUT(pin_mosi, pin_sck, SPI#MSBFIRST , 24, address)   'send address
  SPI.SHIFTOUT(pin_mosi, pin_sck, SPI#MSBFIRST , 8, data&$FF)   'send data
  SPI.SHIFTOUT(pin_mosi, pin_sck, SPI#MSBFIRST , 8, (data>>8)&$FF)   'send data
  high(pin_csn)

PRI WriteReg32(Address,data) '32 bits
  address|=$800000
  low(pin_csn)
  SPI.SHIFTOUT(pin_mosi, pin_sck, SPI#MSBFIRST , 24, address)   'send address
  SPI.SHIFTOUT(pin_mosi, pin_sck, SPI#MSBFIRST , 8, data&$FF)   'send data
  SPI.SHIFTOUT(pin_mosi, pin_sck, SPI#MSBFIRST , 8, (data>>8)&$FF)   'send data
  SPI.SHIFTOUT(pin_mosi, pin_sck, SPI#MSBFIRST , 8, (data>>16)&$FF)   'send data    
  SPI.SHIFTOUT(pin_mosi, pin_sck, SPI#MSBFIRST , 8, (data>>24)&$FF)   'send data    
  high(pin_csn)    

PRI ReadReg8(Address)|data '8-bits
  low(pin_csn)
  SPI.SHIFTOUT(pin_mosi, pin_sck, SPI#MSBFIRST , 32, address<<8)   'send address + dummy byte
  data:=SPI.SHIFTIN(pin_miso, pin_sck, SPI#MSBPRE, 8) 'read data
  high(pin_csn)
  return data
  
PRI ReadReg16(Address)|data '16 bits
  low(pin_csn)
  SPI.SHIFTOUT(pin_mosi, pin_sck, SPI#MSBFIRST , 32, address<<8)   'send address + dummy byte
  data:=SPI.SHIFTIN(pin_miso, pin_sck, SPI#MSBPRE, 8) 'read data
  data+=SPI.SHIFTIN(pin_miso, pin_sck, SPI#MSBPRE, 8)<<8 'read data  
  high(pin_csn)
  return data  

PRI ReadReg32(Address)|data '32 bits
  low(pin_csn)
  SPI.SHIFTOUT(pin_mosi, pin_sck, SPI#MSBFIRST , 32, address<<8)   'send address + dummy byte
  data:=SPI.SHIFTIN(pin_miso, pin_sck, SPI#MSBPRE, 8) 'read data
  data+=SPI.SHIFTIN(pin_miso, pin_sck, SPI#MSBPRE, 8)<<8 'read data  
  data+=SPI.SHIFTIN(pin_miso, pin_sck, SPI#MSBPRE, 8)<<16 'read data  
  data+=SPI.SHIFTIN(pin_miso, pin_sck, SPI#MSBPRE, 8)<<24 'read data  
  high(pin_csn)
  return data

PUB HIGH(Pin)
    dira[Pin]~~
    outa[Pin]~~
         
PUB LOW(Pin)
    dira[Pin]~~
    outa[Pin]~  
      